From c8288abde58aa46ff013772365d6dbe9839ae88d Mon Sep 17 00:00:00 2001
From: Pierre-Louis Bossart <pierre-louis.bossart@intel.com>
Date: Fri, 19 Nov 2010 11:28:40 -0600
Subject: [PATCH 2/2] pcm_plugin: fix capture delay

The existing code only reported the slave delay. Add additional
buffering done at the plugin level.
This makes sure delay >= avail, otherwise the values reported
by snd_pcm_avail_delay don't make any sense.

Signed-off-by: Pierre-Louis Bossart <pierre-louis.bossart@intel.com>
---
 src/pcm/pcm_plugin.c |   14 +++++++++++++-
 1 files changed, 13 insertions(+), 1 deletions(-)

diff --git a/src/pcm/pcm_plugin.c b/src/pcm/pcm_plugin.c
index 19d0dee..2a7934b 100644
--- a/src/pcm/pcm_plugin.c
+++ b/src/pcm/pcm_plugin.c
@@ -141,11 +141,23 @@ static int snd_pcm_plugin_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 {
 	snd_pcm_plugin_t *plugin = pcm->private_data;
 	snd_pcm_sframes_t sd;
-	int err = snd_pcm_delay(plugin->gen.slave, &sd);
+        snd_pcm_sframes_t slave_avail;
+        snd_pcm_sframes_t plugin_avail;
+        int err = snd_pcm_delay(plugin->gen.slave, &sd);
 	if (err < 0)
 		return err;
+        if (pcm->stream == SND_PCM_STREAM_CAPTURE &&
+	    pcm->access != SND_PCM_ACCESS_RW_INTERLEAVED &&
+	    pcm->access != SND_PCM_ACCESS_RW_NONINTERLEAVED)
+		goto _capture;
 	*delayp = sd;
 	return 0;
+ _capture:
+        slave_avail = snd_pcm_avail_update(plugin->gen.slave);
+        plugin_avail = snd_pcm_avail_update(pcm);
+        assert( plugin_avail >= slave_avail);
+        *delayp = sd + (plugin_avail - slave_avail);
+        return 0;
 }
 
 static int snd_pcm_plugin_prepare(snd_pcm_t *pcm)
-- 
1.7.2.3

